/// <reference types="node" />
import { Message, PermissionFlags, Snowflake } from 'discord.js';
import { SilentClient } from '../client/SilentClient';
import Timer = NodeJS.Timer;
interface ICommandOptions {
    commandName: string;
    alias?: string[];
    only: IOnlyOptions;
    ratelimit?: {
        times: number;
        timeout: number;
        all: boolean;
    };
    extendedVerify?: (msg: Message, client: any) => boolean | Promise<boolean>;
    userPermissions: PermissionFlags[];
    restricted: {
        roles: Snowflake[];
        users: Snowflake[];
    };
    desc: string;
    usage: string;
    ownerOnly: boolean;
    nsfw: boolean;
    minRole: string | Snowflake;
    hidden: boolean;
}
interface IOnlyOptions {
    dm: boolean;
    guild: boolean;
    [id: string]: any;
}
/**
 * The Command class to create and initialise new commands
 * @abstract
 */
export declare abstract class Command<S extends SilentClient = SilentClient> {
    /**
     * @property {String} commandName - The name of the command
     * @public
     */
    commandName: string;
    /**
     * @property {String[]} alias - An array of command aliases
     * @public
     */
    alias: string[];
    /**
     * @property {IOnlyOptions} only - Object of restricting attributes
     * @public
     */
    only: IOnlyOptions;
    /**
     * @namespace
     * @property {Object} ratelimit - Holding information about the ratelimit options
     * @property {Number} ratelimit.times - How often a command can be used within x interval
     * @property {Number} ratelimit.timeout - The interval between clearing the data
     * @property {Boolean} ratelimit.all - Weather to include owners in the ratelimit
     * @public
     */
    ratelimit: {
        times: number;
        timeout: number;
        all: boolean;
    };
    /**
     * @property {?Function} extendedVerify - An extending verify function
     * @public
     */
    extendedVerify: (msg: Message, client: S) => boolean | Promise<boolean>;
    /**
     * @property {PermissionFlags[]} userPermissions - Holding required permissions for users to have
     * @public
     */
    userPermissions: PermissionFlags[];
    /**
     * @property {{roles: Snowflake[], users: Snowflake[]}} restricted - information about how is able to use a cmd
     * @public
     */
    restricted: {
        roles: Snowflake[];
        users: Snowflake[];
    };
    /**
     * @property {string} desc - Description of command
     */
    desc: string;
    /**
     * @property {string} usage - Usage of command
     */
    usage: string;
    /**
     * @property {boolean} ownerOnly
     */
    ownerOnly: boolean;
    /**
     * @property {boolean} nsfw
     */
    nsfw: boolean;
    /**
     * @property {string|Snowflake} minRole
     */
    minRole: string | Snowflake;
    /**
     * @property {boolean} hidden
     */
    hidden: boolean;
    /**
     * This is made for internal usage only.
     * @property {String} _className
     * @private
     */
    _className: string;
    /**
     * @property {Object} client - The client instance
     * @protected
     */
    protected client: S;
    /**
     * @namespace ThrottleObject
     * @property {Number} ThrottleObject.start - Start of the throttling process
     * @property {Number} ThrottleObject.uses - Command uses since start
     * @property {Timer} ThrottleObject.timeout - The timeout function to delete this object after the interval
     */
    /**
     * @property {Map<Snowflake, ThrottleObject>} - Storage of the ratelimit
     * @protected
     */
    protected throttled: Map<Snowflake, {
        start: number;
        uses: number;
        timeout: Timer;
    }>;
    protected constructor(options?: ICommandOptions);
    /**
     * Initializes the command to set all needed properties and check the configurations for errors
     * @param client - The client object
     * @returns {Command}
     * @private
     */
    _init(client: S): this;
    /**
     * Get the current status of a user's ratelimit
     * @param userID {Snowflake} The user ID of the person to limit
     * @returns {ThrottleObject}
     * @public
     */
    throttle(userID: Snowflake): {
        start: number;
        uses: number;
        timeout: Timer;
    };
    /**
     * This is used to verify that the command is a valid command and all requirements for it are met
     * You can customize this with the @VerifyExtend decorator
     * @param msg {Message}
     * @param [client] {Object} The client object
     * @returns {Promise<Boolean>}
     * @public
     */
    verify(msg: Message, client?: S): Promise<boolean>;
    /**
     * Run the command
     * @abstract
     * @param values {*[]}
     * @public
     */
    abstract run(...values: any[]): any;
}
export {};
