"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * The Command class to create and initialise new commands
 * @abstract
 */
class Command {
    constructor(options) {
        /**
         * @namespace ThrottleObject
         * @property {Number} ThrottleObject.start - Start of the throttling process
         * @property {Number} ThrottleObject.uses - Command uses since start
         * @property {Timer} ThrottleObject.timeout - The timeout function to delete this object after the interval
         */
        /**
         * @property {Map<Snowflake, ThrottleObject>} - Storage of the ratelimit
         * @protected
         */
        this.throttled = new Map();
        if (options) {
            if (options.commandName)
                this.commandName = options.commandName;
            if (options.alias)
                this.alias = options.alias;
            if (options.only)
                this.only = options.only;
            else
                this.only = { guild: false, dm: false };
            if (options.extendedVerify)
                this.extendedVerify = options.extendedVerify;
            if (options.ratelimit)
                this.ratelimit = options.ratelimit;
            if (options.userPermissions)
                this.userPermissions = options.userPermissions;
            if (options.restricted)
                this.restricted = options.restricted;
            if (options.desc)
                this.desc = options.desc;
            if (options.usage)
                this.usage = options.usage;
            if (options.ownerOnly)
                this.ownerOnly = true;
            if (options.nsfw)
                this.nsfw = true;
            if (options.minRole)
                this.minRole = options.minRole;
            if (options.hidden)
                this.hidden = true;
        }
    }
    /**
     * Initializes the command to set all needed properties and check the configurations for errors
     * @param client - The client object
     * @returns {Command}
     * @private
     */
    _init(client) {
        this.client = client;
        if (!this.commandName)
            throw new TypeError('Cannot initiate command without a name.');
        if (this.alias === undefined)
            this.alias = [];
        return this;
    }
    /**
     * Get the current status of a user's ratelimit
     * @param userID {Snowflake} The user ID of the person to limit
     * @returns {ThrottleObject}
     * @public
     */
    throttle(userID) {
        if (!this.ratelimit.all && (this.client.owner.includes(userID) || !this.ratelimit))
            return null;
        let throttleObj = this.throttled.get(userID);
        if (!throttleObj) {
            throttleObj = {
                start: Date.now(), uses: 0, timeout: this.client.setTimeout(() => {
                    this.throttled.delete(userID);
                }, this.ratelimit.timeout * 1000)
            };
            this.throttled.set(userID, throttleObj);
        }
        return throttleObj;
    }
    /**
     * This is used to verify that the command is a valid command and all requirements for it are met
     * You can customize this with the @VerifyExtend decorator
     * @param msg {Message}
     * @param [client] {Object} The client object
     * @returns {Promise<Boolean>}
     * @public
     */
    verify(msg, client) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!client && !this.client)
                throw new Error('Can not use verify without client.');
            if (client)
                this.client = client;
            if (this.ownerOnly && msg.author.id !== client.owner)
                return false;
            if (this.extendedVerify !== undefined) {
                let extendedResult = this.extendedVerify(msg, this.client);
                if (extendedResult.constructor.name === 'Promise') {
                    extendedResult = yield extendedResult;
                }
                if (!extendedResult)
                    return;
            }
            if (this.restricted &&
                (!this.restricted.users.includes(msg.author.id) ||
                    msg.guild && !msg.member.roles.some(r => this.restricted.roles.includes(r.id))))
                return false;
            if (this.ratelimit) {
                const throttle = this.throttle(msg.author.id);
                if (throttle && throttle.uses + 1 > this.ratelimit.times) {
                    if (this.client.rateLimitExceededError) {
                        const timeout = (throttle.start + (this.ratelimit.timeout * 1000) - Date.now()) / 1000;
                        yield msg.reply(`This command is throttled for another ${timeout.toFixed(2)} seconds.`);
                    }
                    return false;
                }
            }
            if (msg.guild && this.userPermissions && !msg.member.hasPermissions(this.userPermissions))
                return false;
            if (msg.guild && !msg.channel.nsfw && this.nsfw)
                return false;
            if (msg.guild && this.minRole) {
                const role = msg.guild.roles.find(r => r.name.toLowerCase() === this.minRole.toLowerCase() ||
                    r.id === this.minRole);
                if (msg.member.highestRole.comparePositionTo(role) < 0) {
                    return false;
                }
            }
            if (this.only.guild && !msg.guild)
                return false;
            if (this.only.dm && msg.guild)
                return false;
            const temp = [];
            let standardLimitOnly = true;
            for (const channel in this.only) {
                if (['guild', 'dm'].includes(channel))
                    continue;
                standardLimitOnly = false;
                temp.push(this.only[channel]);
            }
            return temp.includes(msg.channel.id) || standardLimitOnly;
        });
    }
}
exports.Command = Command;
//# sourceMappingURL=Command.js.map